function [A,b,K,w,dv2x,Nfv,feas] = sospsimplify(A,b,K,w,dv2x,Nsosvarc);
% function [A,b,K,w,dv2x,Nfv] = sospsimplify(A,b,K,w,dv2x,Nsosvarc);
%
% DESCRIPTION 
%   This function performs a simplification procedure on the SOS problem.
%   First, it tries to detect the sign of optimization variables based on
%   simple constraints. Second, it searches for monomials that can be 
%   removed from each SOS constraint. This search is based on diagonal 
%   entries of the Gram matrix that are forced to be zero and it is 
%   equivalent to the Newton Polytope method. These two steps are repeated
%   until no new sign information can be detected.
%
%   In the code, the information about the sign of the optimization 
%   variables is stored in xsign where:
%      xsign(i)=NaN    if x(i) has unknown sign
%      xsign(i)=+1     if x(i)>=0
%      xsign(i)=-1     if x(i)<=0
%      xsign(i)=0      if x==0
%
% 9/21/08  PJS     Initial Coding


%--------------------------------------------------------------------
% Grab problem dimensions
%--------------------------------------------------------------------
Nfv = K.f;
Nlp = K.l;
Nsv = sum( K.s(1:Nsosvarc).^2 );
Nx = size(A,2);

%--------------------------------------------------------------------
% Find non-negative optimization variables
%--------------------------------------------------------------------
xsign = repmat(NaN,[Nx,1]);

% Process LP constraints: A*d+y=b, y>=0
if Nlp>0
    % LP slack vars are >= 0
    xsign(Nfv+1:Nfv+Nlp) = +1;
end

% Process SOS Inequality Constraints: Ad*d + Aq*Q(:) = b and Q>=0
ptr = Nfv+Nlp;
for i1=1:length(K.s)
    % Diag entries of Q are >= 0    
    lw = K.s(i1);
    diagidx = (0:lw-1)*lw+(1:lw);
    xsign(ptr+diagidx) = +1;
    ptr = ptr+lw^2;
end

%--------------------------------------------------------------------
% Find dec vars and monomials that can be removed
%--------------------------------------------------------------------
go = 1;
xsignprev = xsign;
while (go == 1)
    % Use simple constraints to determine sign of optimization vars. 
    xsign = LOCALxsignupdate(xsign,A,b);
    A( : , xsign==0 ) = 0;
    
    % Monomial reduction procedure 
    % (This is equivalent to the Newton Polytope method)
    ptr = Nfv+Nlp;
    for i1=1:length(K.s)
        % Find diag entries of Q that are forced to be zero
        lw = K.s(i1);
        blkidx = ptr+(1:lw^2);
        Qsign = diag( reshape(xsign(blkidx),[lw lw]) );
        loc = find(Qsign==0);
        
        if ~isempty(loc)
            % Corresponding rows/cols of zero diag entries are also zero
            tmp = sparse(lw,lw);
            tmp(loc,:) = 1;
            tmp(:,loc) = 1;
            rmidx = find(tmp)+ptr;
            xsign(rmidx) = 0;    
            
            % Remove vars/monoms associated with zero Gram matrix entries
            A(:,rmidx) = [];
            xsign(rmidx) = [];
            w{i1}(loc) = [];
            K.s(i1) = length( w{i1} );
            
            % Update the mapping of dec vars into the optimization vars
            if i1<=Nsosvarc
                % Number of optim vars to remove
                Nremove = length(rmidx);
                
                % Optim vars currently in this block (before removal)
                blkidx = ptr+(1:lw^2);

                % Mark removed dec vars
                dv2x( ismember(dv2x,rmidx) ) = 0;
                          
                % Relabel the remaining dec vars in this block
                idx = find( tril(ones(K.s(i1))) );                            
                dv2x( ismember(dv2x,blkidx) ) = ptr+idx;
                                
                % Relabel remaining dec vars in subsequent blocks
                idx = find( dv2x>blkidx(end) );
                dv2x(idx) = dv2x(idx) - Nremove;                                               
            end        
        end
        
        % Update pointer
        ptr = ptr+K.s(i1)^2;
    end

    % Continue if xsign has been updated
    go = ~isequalwithequalnans(xsign,xsignprev);
    xsignprev = xsign;    
end

%--------------------------------------------------------------------
% Clean up 
%--------------------------------------------------------------------

% Mark removed free decision vars
rmidx = find( xsign(1:K.f)==0 );
dv2x( ismember(dv2x,rmidx) ) = 0;
idx = find(dv2x<=K.f & dv2x>0);
A(:,rmidx) = [];
xsign(rmidx) = [];
K.f = K.f - length(rmidx);  
Nfv = K.f;
dv2x(idx) = 1:Nfv;

% Remove any constraints of the form 0=0 
% XXX -- We need a tolerance here.  How should we choose tol?
tol = 1e-9;
ridx = find( sum(A~=0,2)==0 & abs(b)<tol*max(abs(b)) );
A(ridx,:) = [];
b(ridx) = [];

% Check for infeasible problems of the form 0 = bi where
% bi is not equal to zero (Our simplify code can uncover
% such infeasibilities, but Sedumi might error out)
ridx = find( sum(A~=0,2)==0 & abs(b)>tol*max(abs(b)) );
feas = 1;
if ~isempty(ridx)
    feas = 0;
end

% Add equality constraints for remaining dec vars known to be zero
idx = find( xsign==0 );
lidx = length(idx);
A(end+1:end+lidx,idx) = speye(lidx);
b = [b; sparse(lidx,1)];



%--------------------------------------------------------------------
% Local function to update sign of optimization var 
%--------------------------------------------------------------------
function xsign = LOCALxsignupdate(xsignOld,A,b)

% Initialize output
xsign = xsignOld;

% Process constraints of the form:  aij*xj = bi
ridx = find(  sum(A~=0,2)==1  );
[cidx,tmp]=find( A(ridx,:)' );
idx = sub2ind(size(A),ridx,cidx);

signA = sign( A(idx) );
signb = sign( b(ridx) );    
xsignUpdate = signA.*signb;
xsign(cidx) = LOCALupdate(xsign(cidx),xsignUpdate);

% Process constraints of the form:  aij*xj + aik*xk = bi
ridx = find(  sum(A~=0,2)==2  );
[cidx,tmp]=find( A(ridx,:)' );
cidx = reshape(cidx,[2 length(ridx)])';
cidx1 = cidx(:,1);
idx1 = sub2ind(size(A),ridx,cidx1);
cidx2 = cidx(:,2);
idx2 = sub2ind(size(A),ridx,cidx2);

c1 = b(ridx)./A(idx1);
c2 = -A(idx2)./A(idx1);
xsignUpdate = repmat(NaN,[length(ridx) 1]);
xsignUpdate( c1<=0 & (c2.*xsign(cidx2)<=0) ) = -1;
xsignUpdate( c1>=0 & (c2.*xsign(cidx2)>=0) ) = +1;
xsign(cidx1) = LOCALupdate(xsign(cidx1),xsignUpdate);

c1 = b(ridx)./A(idx2);
c2 = -A(idx1)./A(idx2);
xsignUpdate = repmat(NaN,[length(ridx) 1]);
xsignUpdate( c1<=0 & (c2.*xsign(cidx1)<=0) ) = -1;
xsignUpdate( c1>=0 & (c2.*xsign(cidx1)>=0) ) = +1;
xsign(cidx2) = LOCALupdate(xsign(cidx2),xsignUpdate);

%--------------------------------------------------------------------
% Local function to update sign of optimization var
%--------------------------------------------------------------------
function xsignNew = LOCALupdate(xsign,xsignUpdate)

% Find constraints that force ai=0, ai<0 and ai>0
zidx = find( (xsignUpdate==0) | (xsign==-1 & xsignUpdate>0) | ...
    (xsign==+1 & xsignUpdate<0) );
nidx = find( isnan(xsign) & xsignUpdate<0 );
pidx = find( isnan(xsign) & xsignUpdate>0 );

% Update xsign
xsignNew = xsign;
xsignNew( zidx ) = 0;
xsignNew( nidx ) = -1;
xsignNew( pidx ) = +1;


