function [lz,lw,h] = sossize(n,d)
% function [lz,lw,h] = sossize(n,d);
%
% DESCRIPTION 
%   This function computes the size of an SOS LMI feasibility problem:
%      p is SOS iff there is a Q>=0 such that p = z^T*Q*z
%   z is an lz-by-1 vector of monomials and Q is an lz-by-lz symmetric
%   matrix.   This can solved as an LMI in two ways:
%     Primal Form:  Find Q>=0 such A*vec(Q) = c where A is an lw-by-lz^2
%       matrix and c is an lw-by-1 vector.  A and c are obtained by
%       equating coefficients of p=z^T*Q*z
%     Dual Form:  Find lambda in R^h such that Q0+ sum_i lambda_i*N*>=0.
%       Q0 is a particular solution to p = z^T*Q*z and the N_i are a 
%       basis of homogeneous solutions satisfying z^T*Ni*z = 0.
%   This function computes the dimensions lz, lw, and h as a function
%   of the polynomial degree and number of variables.
%
% INPUTS
%   n: Number of variables in the polynomial
%   d: Degree of polynomial. Note--If d is odd then the Gram matrix
%     representation requires all monomials of degree up to ceil(d/2).
%   n,d: p is a polynomial in n variables of degree 2d
%
% OUTPUTS
%   lz: Number of monomials of degree ceil(d/2) in n variables.  This is
%      the length of z and dimension of Q in the Gram matrix
%      representation.
%   lw: Number of monomials of degree d in n variables.  This is the
%      number of equality constraints in the primal LMI form.
%   h:  Number of linearly independent homogenous solutions satisfying 
%      z^T N z = 0.  
%
% SYNTAX 
%   [lz,lw,h] = sossize(n,d)
%

% 9/26/09   PJS     Initial Coding

% Number of monomials in z
dhalf = ceil(d/2);
lz = nchoosek(n+dhalf,dhalf);

% Number of monomials in w
lw = nchoosek(n+d,d);

% Qdim is the number of unique entries in the symmetric matrix Q
Qdim = lz*(lz+1)/2;  
h = Qdim - lw;

